package mt

import (
	"q5"
	"f5"
	"fmt"
	"main/constant"
)

type Currency struct {
	currencyName string
	exchangeRate int64
	currencyDecimal int64
}

type CurrencyTable struct {
	f5.CustomMetaTable
	netIdHash *q5.ConcurrentMap[int32, *Currency]
}

func (this *Currency) init(currencyName string, exchangeRate int64, currencyDecimal int64) {
	this.currencyName = currencyName
	this.exchangeRate = exchangeRate
	this.currencyDecimal = currencyDecimal
}

func (this *Currency) GetCurrencyName() string {
	return this.currencyName
}

func (this *Currency) GetExchangeRate() int64 {
	return this.exchangeRate
}

func (this *Currency) GetCurrencyDecimal() int64 {
	return this.currencyDecimal
}

func (this *Currency) check() {
	if this.GetExchangeRate() <= 0 {
		panic("Currency exchange_rate <= 0")
		return
	}
	if this.GetExchangeRate() != constant.RECHARGE_CURRENCY_MAX_EXCHANGE_RAET {
		panic("Currency exchange_rate > uplimit")
		return
	}
	if this.GetCurrencyDecimal() <= 0 {
		panic("Currency currency_decimal <= 0")
		return
	}
	if this.GetCurrencyDecimal() != constant.RECHARGE_CURRENCY_MAX_DECIMAL {
		panic("Currency exchange_rate > uplimit")
		return
	}
}

func (this *CurrencyTable) Get(netId int32) *Currency {
	if v, ok := this.netIdHash.Load(netId); ok {
		return *v
	} else {
		return nil
	}
}

func (this *CurrencyTable) Load() {
	this.netIdHash = new(q5.ConcurrentMap[int32, *Currency])
	nets := []int32{}
	{
		if jsonStr, err := f5.ReadJsonFile("../config/nets.json"); err == nil {
			if err := q5.DecodeJson(jsonStr, &nets); err != nil {
				panic(fmt.Sprintf("load metafile json decode error %s %s", "nets.json", err))
			}
		} else {
			panic(fmt.Sprintf("load metafile error %s %s", "nets.json", err))
		}
	}
	{
		for _, val := range nets {
			netId := val
			fileName := fmt.Sprintf("../config/currencys/%d/currency.json", netId)
			if jsonStr, err := f5.ReadJsonFile(fileName); err == nil {
				f5.GetSysLog().Info("load currency %s", fileName)
				currencyCfg := struct {
					CurrencyName string `json:"currency_name"`
					ExchangeRate int64 `json:"exchange_rate"`
					CurrencyDecimal int64 `json:"currency_decimal"`
				}{}
				if err := q5.DecodeJson(jsonStr, &currencyCfg); err != nil {
					panic(fmt.Sprintf("load metafile json decode error %s %s", "currency.json", err))
				}
				p := new(Currency)
				p.init(currencyCfg.CurrencyName, currencyCfg.ExchangeRate, currencyCfg.CurrencyDecimal)
				p.check()
				this.netIdHash.Store(netId, p)
			}
		}
	}
}

func (this *CurrencyTable) PostInit1() {
	this.netIdHash.Range(
		func (key int32, val *Currency) bool {
			netId := key
			currencyMeta := val
			Table.Recharge.Traverse(func(ele *Recharge) bool {
				if int64(ele.GetPrice()) * currencyMeta.GetExchangeRate() != int64(ele.GetDiamond()) {
					panic(fmt.Sprintf("Currency verifyerror net_id:%d id:%d", netId, ele.GetId()))
				}
				return true
			})
			return true
		})
}
